import re, os, sys, string
import SCons.Util
from SCons.Options import Options


def exists(env):
	return True


def CheckPKGConfig(context, version):
	context.Message( 'Checking for pkg-config >= %s... ' %version )
	ret = context.TryAction('pkg-config --atleast-pkgconfig-version=%s' % version)[0]
	context.Result( ret )
	return ret

def CheckPKG(context, name, version):
	context.Message( 'Checking for %s >= %s ... ' % (name,version) )
	ret = context.TryAction('pkg-config --atleast-version=%s \'%s\'' % (version,name))[0]
	context.Result( ret )
	return ret


def generate(env):

	usrcachefile = 'usropts.py'
	intcachefile = 'intopts.py'

	usropts = Options(usrcachefile)
	intopts = Options(intcachefile)

	#user visible options
	usropts.AddOptions(
		('debug',             'Set to yes to produce a binary with debug information', 0),
		('optimize',          'Enable processor optimizations during compilation', 2),
		('profile',           'Set to yes to produce a binary with profiling information', False),
		('strip',             'Discard symbols from the executable (only when neither debugging nor profiling)', True))

	#internal options
	intopts.AddOptions(
		('LINKFLAGS',     'linker flags'),
		('LIBPATH',       'library path'),
		('LIBS',          'libraries'),
		('CCFLAGS',       'c compiler flags'),
		('CXXFLAGS',      'c++ compiler flags'),
		('CPPDEFINES',    'c preprocessor defines'),
		('CPPPATH',       'c preprocessor include path'),
		('is_configured', 'configuration version stamp'))

	usropts.Update(env)
	intopts.Update(env)

	env.Help(usropts.GenerateHelpText(env))

	# Use this to avoid an error message 'how to make target configure ?'
	env.Alias('configure', None)

	if not 'configure' in sys.argv and not ((env.has_key('is_configured') and env['is_configured'] == 1) or env.GetOption('clean')):
		print "Not configured or configure script updated.  Run `scons configure' first."
		print "Use `scons --help' to show available configure options to `scons configure'."
		env.Exit(1)

	
	if 'configure' in sys.argv:

		# Important : unset existing variables
		for key in ['debug', 'optimize', 'profile', 'cachedir', 'strip', 'LINKFLAGS', 'LIBPATH', 'LIBS', 'CCFLAGS', 'CXXFLAGS', 'CPPDEFINES', 'CPPPATH', 'is_configured']:
			if env.has_key(key): env.__delitem__(key)

		print "\nNow configuring.  If something fails, consult `config.log' for details.\n"

		#parse cmdline
		def makeHashTable(args):
			table = { }
			for arg in args:
				if len(arg) > 1:
					lst = arg.split('=')
					if len(lst) < 2: continue
					key = lst[0]
					value = lst[1]
					if len(key) > 0 and len(value) > 0: table[key] = value
			return table

		args = makeHashTable(sys.argv)


		env['is_configured'] = 1

		# Declare some helper functions for boolean and string options.
		def bool_opt(key, default):
			if args.has_key(key):
				if args[key] == 'no' or args[key] == 'false' or args[key] == '0':
					env[key] = False
				elif args[key] == 'yes' or args[key] == 'true' or args[key] == '1':
					env[key] = True
				else:
					print "\ninvalid", key, "option, must be one of: yes, true, no, false, 0, 1."
					env.Exit(1)
			else: env[key] = default

		def string_opt(key, default):
			if args.has_key(key):
				env[key] = args[key]
			else: env[key] = default



		# start with empty FLAGS, in case everything is disabled.
		env['CCFLAGS'] = []

		# profile?
		bool_opt('profile', False)
		if env['profile']:
			print "profiling enabled,",
			env.AppendUnique(CCFLAGS=['-pg'], LINKFLAGS=['-pg'])
		else:
			print "profiling NOT enabled,",

		# debug?
		if args.has_key('debug'):
			level = args['debug']
			if level == 'no' or level == 'false': level = '0'
			elif level == 'yes' or level == 'true': level = '3'
		else:
			level = '0'
		if int(level) == 0:
			print "debugging NOT enabled,",
			env['debug'] = 0
		elif int(level) >= 1 and int(level) <= 3:
			print "level", level, "debugging enabled,"
			env['debug'] = level
			env.AppendUnique(CCFLAGS=['-ggdb'+level], CPPDEFINES=['DEBUG'])
		else:
			print "\ninvalid debug option, must be one of: yes, true, no, false, 0, 1, 2, 3."
			env.Exit(1)

		# optimize?
		if args.has_key('optimize'):
			level = args['optimize']
			if level == 'no' or level == 'false': level = '0'
			elif level == 'yes' or level == 'true': level = '2'
		else:
			if env['debug']: level = '0'
			else: level = '2'
		if level == 'noarch':
		 	print "noarch optimizing"
			env['optimize'] = 'noarch'
			env.AppendUnique(CCFLAGS=['-O2', '-pipe'])
		elif int(level) >= 1 and int(level) <= 3:
			print "level", level, "optimizing enabled"
			env['optimize'] = level
			env.AppendUnique(CCFLAGS=['-O'+level, '-pipe'])
		elif int(level) == 0:
			print "optimizing NOT enabled,"
			env['optimize'] = 0
		else:
			print "\ninvalid optimize option, must be one of: yes, true, no, false, 0, 1, 2, 3, noarch."
			env.Exit(1)


		bool_opt('strip', True)
		string_opt('cachedir', None)


		conf = env.Configure(custom_tests = { 'CheckPKGConfig' : CheckPKGConfig,'CheckPKG' : CheckPKG })

		if env['PLATFORM'] == 'win32':

			env.Append(CCFLAGS = ['-I/GTK/include/libxml2/','-I/mingw/include/AL'])
			env.Append(LIBPATH = ['-L/GTK/lib/'])
			env.Append(LIBS = ['-lxml2','-lz','-liconv','-lws2_32','-lsndfile'])
			
			## Check for minGW
			if not conf.CheckLib('mingw32'):
				print 'mingw is required for this program'
				env.Exit(1)

			## Check for openAL 
			if not conf.CheckCHeader('al.h'):
				print "openAL is required for this program"
				env.Exit(1)
			if not conf.CheckLib('openal32'):
				print 'libopenal32 is required for this program'
				env.Exit(1)

#			Doesn't seem to work on my Windows :
#
#			if not conf.CheckPKGConfig("0.15.0"):
#				print 'pkg-config not found.'
#				env.Exit(1)
#							
#								
#			if not conf.CheckPKG('gtkmm-2.4','2.10'):
#				print 'You need the gtkmm-2.4 development packages'
#				env.Exit(1)a


			# When using ParseConfig with pkg-config, scons will use CPPPATH and scan _every_ path, which takes a lot of time.
			# When using the CCFLAGS it doesn't scan paths and it's really a lot faster.
			# Therefore what we do here is getting the result of pkg-config --cflags for our libraries and store the result into CCFLAGS

			pipe = os.popen('pkg-config --cflags gtkmm-2.4 gthread-2.0')
			output = pipe.read()
			retcode = pipe.close()
			env.Append(CCFLAGS = string.split(output.rstrip('\n')))


			# For the LINKERFLAGS we can use ParseConfig, libs don't impact on performance

			env.ParseConfig('pkg-config --libs gtkmm-2.4 gthread-2.0')


		else:

			if not conf.CheckPKGConfig("0.15.0"):
				print 'pkg-config not found.'
				env.Exit(1)
							
								
			if not conf.CheckPKG('gtkmm-2.4','2.10'):
				print 'You need the gtkmm-2.4 development packages'
				env.Exit(1)				
				
				
			if not conf.CheckPKG('openal','0.0.8'):
				print 'You need openal development packages'
				env.Exit(1)


			if not conf.CheckPKG('sndfile','1.0.15'):
				print 'You need libsndfile development packages'
				env.Exit(1)

                        pipe = os.popen('pkg-config --cflags gtkmm-2.4 openal gthread-2.0 sndfile')
                        output = pipe.read()
                        retcode = pipe.close()
                        env.Append(CCFLAGS = string.split(output.rstrip('\n')))


			# When using ParseConfig with pkg-config, scons will use CPPPATH and scan _every_ path, which takes a lot of time.
			# When using the CCFLAGS it doesn't scan paths and it's really a lot faster.
			# Therefore what we do here is getting the result of pkg-config --cflags for our libraries and store the result into CCFLAGS

			pipe = os.popen('pkg-config --cflags gtkmm-2.4 openal gthread-2.0 sndfile')
			output = pipe.read()
			retcode = pipe.close()
			env.Append(CCFLAGS = string.split(output.rstrip('\n')))

			pipe = os.popen('xml2-config --cflags')
			output = pipe.read()
			retcode = pipe.close()
			env.Append(CCFLAGS = string.split(output.rstrip('\n')))



			# For the LINKERFLAGS we can use ParseConfig, libs don't impact on performance

			env.ParseConfig('pkg-config --libs gtkmm-2.4 openal gthread-2.0 sndfile')
			env.ParseConfig('xml2-config --libs')


		
		env = conf.Finish()
		
		print "========================================================="
		print "\nEverything seems OK.  Run `scons' now to build.\n"
		print "========================================================="


		usropts.Save(usrcachefile, env)
		intopts.Save(intcachefile, env)

	#Should we strip the exe?
	if env.has_key('strip') and env['strip'] and not env['debug'] and not env['profile'] and not env.GetOption('clean'):
		env['strip'] = True
	else:
		env['strip'] = False



