#include "audioIO.h"
#include <AL/al.h>
#include <AL/alc.h>
#include "sndfile.h"
#include <iostream>
#include <fstream>

using namespace std;



AudioIO::AudioIO()
{
	audioInitOK = false;
	al_source = 0;
	al_buffer = 0;
	output_stream_data = NULL;
	nb_frames = 0;
}


AudioIO::~AudioIO()
{
	terminateAudio();
}


bool AudioIO::initAudio()
{
	audioInitOK = false;
	
	ALCdevice* device = alcOpenDevice(NULL);
	if (!device)
		return false;

	ALCcontext* context = alcCreateContext(device, NULL);
	if (!context)
	{
		alcCloseDevice(device);
		return false;
	}


	if (!alcMakeContextCurrent(context))
	{
		alcDestroyContext(context);
		alcCloseDevice(device);
		return false;
	}
	
	alGenBuffers(1, &al_buffer);
	alGenSources(1, &al_source);

	audioInitOK = true;
	return true;
}


void AudioIO::terminateAudio()
{
	if (!audioInitOK)
		return;
	
	stopSound();
	
	alDeleteBuffers(1, &al_buffer);
	alDeleteSources(1, &al_source);
	
	ALCcontext* context = alcGetCurrentContext();
	ALCdevice*  device  = alcGetContextsDevice(context);
	
	alcMakeContextCurrent(NULL);
	alcDestroyContext(context);
	alcCloseDevice(device);
}


bool AudioIO::loadSound(const string& filename)
{
	SF_INFO info;
	info.format = 0;
	SNDFILE* file = sf_open(filename.c_str(), SFM_READ, &info);
	
	if (!file)
		return false;
	
	int nb_samples  = info.channels * info.frames;
	
	sample_rate = info.samplerate;
	nb_channels = info.channels;
	nb_frames = info.frames;
	
	if (output_stream_data)
	{
		delete [] output_stream_data;
		output_stream_data = NULL;
	}
	
	output_stream_data = new OUTPUT_SAMPLE[nb_samples];
	
	if (sf_read_short(file, output_stream_data, nb_samples) < nb_samples)
		return false;
	
	sf_close(file);
	
	// find the max value
	max_value = 0;
	for(int i=0;i<nb_samples;i++)
		if (abs(output_stream_data[i]>max_value)) 
			max_value=output_stream_data[i];
	
	return true;
}

void AudioIO::unloadSound()
{
	if (is_playing()) return;
	
	if (output_stream_data)
	{
		delete [] output_stream_data;
		output_stream_data = NULL;
	}
}

void AudioIO::setSequenceToPlay(double begin, double end)
{
	if (is_playing()) return;
	
	output_position = (int)(begin*(double)nb_frames);
	output_end = (int)(end*(double)nb_frames);
	
	if (output_position>output_end)
	{
		int tmp = output_position;
		output_position = output_end;
		output_end = tmp;
	}
	
	if (output_position<0) output_position=0;
	if (output_end > nb_frames) output_end = nb_frames;
	
	
	// Set up openAl
	
	ALenum format;
	switch (nb_channels)
	{
		case 1 :  format = AL_FORMAT_MONO16;   break;
		case 2 :  format = AL_FORMAT_STEREO16; break;
		default : format = AL_FORMAT_MONO16;   break;
	}
	
	alSourcei(al_source, AL_BUFFER, 0);
	alBufferData(al_buffer, format, output_stream_data + output_position, (output_end - output_position) * sizeof(ALushort), sample_rate);

	if (alGetError() != AL_NO_ERROR)
	{
		cout << "OpenAL : Error " << alGetError() << endl;
	}

}


void AudioIO::playSound()
{
	if (is_playing()) return;
	
	alSourcei(al_source, AL_BUFFER, al_buffer);	
	alSourcePlay(al_source);
}


void AudioIO::stopSound()
{
	if (is_playing())
		alSourceStop(al_source);
}

bool AudioIO::is_playing()
{
	ALint status;
	alGetSourcei(al_source, AL_SOURCE_STATE, &status);
	return (status == AL_PLAYING);
}

double AudioIO::getPosition(int elapsed_time)
{
	if (is_playing()==false) return 0.0;

	return (((elapsed_time / 1000.0) * sample_rate) / (nb_frames / nb_channels));

}


