#ifndef __HTK_GRAPH__
#define __HTK_GRAPH__

#include <string>
#include <list>
#include <vector>
#include <map>



const std::string  EXPORT_FORMAT	=	"png";
const std::string  GRAPH_IMAGE_IMG	=	"graph." + EXPORT_FORMAT;
const std::string  GRAPH_IMAGE_DOT	=	"graph.dot";


class Graphe;


/** This struct is used to describe an element of the graph.
  * The element can be a node or an edge.
  * Mainly used when drawing the elements.
  */
struct GraphElement
{
	int id;				//!< Identifier of the edge
	int x;				//!< x position of the element
	int y;				//!< y position of the element
	int width;			//!< width of the element (only for the node)
	int height;			//!< height of the element (only for the node)
	int number;			//!< If the element is a node, it represents the node's number
	int from;			//!< If the element is an edge, it is the first node
	int dest;			//!< If the element is an edge, it is the second node
	double time;		//!< Only for a node, represents the normalized time in the audio stream
	bool node;			//!< True if the element is a node, false if it is an edge
	std::string label;	//!< Label of the edge
	std::string XMLData;//!< XML data associated to this edge
	double value;		//!< Value of the edge
};


struct HTKNode
{
	double time;
	std::map<std::string, double> values;
};

struct XMLAttrPos
{
	std::vector<int> attr_pos;
	std::string attr_name;
};


class HTKGraph
{
	public:
		HTKGraph();
		~HTKGraph();
		
		std::string parseHTK(const std::string& filename);
		bool createGraph(const char* export_filename=0);
		void displayShortestPath(bool display);
		void displayShortestPathByAnEdge(bool display, const GraphElement& _edge_selected);
		bool loadGraphElements(std::list<GraphElement>& graph_elements);
		
		double getDistance();
		double getDistanceByAnEdge();
		
		void takeTheOppositeScore(bool opposite, std::list<GraphElement>& graph_elements);
		bool getOppositeScore(){return score_opposite;}
		
		void DisplayScoreOnEdges(bool score) {display_score = score;}
		void DisplayXMLOnEdges(const std::vector<XMLAttrPos>& _xml_attr) {xml_attr = _xml_attr; m_use_path=false;}
		void DisplaySelectedXMLOnEdges(const std::vector<std::vector<int> >& _xml_path) {xml_path = _xml_path;m_use_path = true;}
		
		void getScores(std::map<std::string, double>& s);
		void set_export_format(const std::string& format) { export_format = format;}

		std::string getShortestPathStr() { return shortest_path_str;}
		std::string getShortestPathByAnEdgeStr() { return shortest_path_by_an_edge_str;}
		
		int dot_width, dot_height; // the size of the bounding box in the dot file
		bool display_shortest_path;
		bool display_shortest_path_by_an_edge;
			
	private:
		double shortestPath(int* path, int* id_list, int n_nodes, bool with_edge = false);

		std::string getXMLInfo(const std::string& xml, bool use_path);
		
		GraphElement edge_selected;
		
		Graphe* graph;
		HTKNode* nodes;
		
		double distance_shortest_path;
		double distance_shortest_path_by_an_edge;
		
		std::string information;
		bool information_available;

		bool display_score;
		std::vector<XMLAttrPos> xml_attr;
		std::vector< std::vector<int> > xml_path;
		bool m_use_path;
		
		bool score_opposite;
		
		std::string export_format;

		std::string shortest_path_str;
		std::string shortest_path_by_an_edge_str;

};





#endif

